import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Widget/baseWidgets.dart';

class BaseWidgetBuilder<T extends BaseController> extends BaseWidget<T> {
  BaseWidgetBuilder({
    @required T controller,
    @required this.builder,
    this.initStateBuilder,
    this.disposeBuilder,
    this.skeletonBuilder,
    this.toStringBuilder,
  }) : super(controller);

  final Widget Function(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    T controller,
  ) builder;
  final Function(GetBuilderState<T> state) initStateBuilder;
  final Function(GetBuilderState<T> state) disposeBuilder;
  final Widget Function(BuildContext context, T controller) skeletonBuilder;
  final String Function(DiagnosticLevel minLevel) toStringBuilder;

  @override
  void initState(GetBuilderState<T> state) {
    super.initState(state);
    initStateBuilder?.call(state);
  }

  @override
  void dispose(GetBuilderState<T> state) {
    super.dispose(state);
    disposeBuilder?.call(state);
  }

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) =>
      toStringBuilder?.call(minLevel) ?? super.toString(minLevel: minLevel);

  @override
  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    BaseController baseController,
  ) {
    return builder?.call(context, data, baseController) ??
        getSkeletonView(context, baseController);
  }

  @override
  Widget getSkeletonView(
    BuildContext context,
    BaseController baseController,
  ) {
    return skeletonBuilder?.call(context, baseController) ??
        super.getSkeletonView(context, baseController);
  }
}
